package provide MODDBC 1.0

proc db_register {db_type initcmd destroycmd connectcmd disconnectcmd sqlcmd \
                  nextcmd colnamecmd coltypecmd colsizecmd} {
  global db_types

  if {[lsearch -exact [array names db_types] $db_type]!=-1} {
    return -code error "$db_type already registered"
  }

  $initcmd

  set db_types($db_type) "$initcmd $destroycmd $connectcmd \
                          $disconnectcmd $sqlcmd $nextcmd $colnamecmd \
                          $coltypecmd $colsizecmd"

}

proc db_unregister {db_type} {
  global db_types
  global handles

  if {[lsearch -exact [array names db_types] $db_type]==-1} {
    return -code error "$db_type not registered"
  }

  [lindex $db_types($db_type) 1]
  unset $db_types($db_type)
}

proc db_connect {db_type db_name host port user password} {
  global db_types
  global handles

  if {[lsearch -exact [array names db_types] $db_type]==-1} {
    return -code error "$db_type not registered"
  }

  set handle [[lindex $db_types($db_type) 2] $db_name $host $port $user $password]
  set handles($handle) $db_type

#  proc $handle {args} {
#    handledbcmd $args
#  }

  return $handle
}

proc db_disconnect {handle} {
  global db_types
  global handles

  set db_type ""

  foreach i [array names handles] {
    if {$i==$handle} {
      set db_type $handles($i)
    }
  }

  if {$db_type!=""} {
    [lindex $db_types($db_type) 3] $handle
  } else {
    return -code error "Handle $handle does not exist"
  }
}

proc handledbcmd {args} {
  set args [lindex $args 0]

  if {[lindex $args 0]=="sql"} {
    db_sql $handle [lindex $args 1]
  } elseif {[lindex $args 0]=="next"} {
    db_next $handle
  } else {
    return -code error "Bad Command"
  }
}

proc db_sql {handle cmd} {
  global db_types
  global handles

  set db_type ""

  foreach i [array names handles] {
    if {$i==$handle} {
      set db_type $handles($i)
    }
  }

  if {$db_type!=""} {
    [lindex $db_types($db_type) 4] $handle $cmd
  } else {
    return -code error "Handle $handle does not exist"
  }
}

proc db_next {handle} {
  global db_types
  global handles

  set db_type ""

  foreach i [array names handles] {
    if {$i==$handle} {
      set db_type $handles($i)
    }
  }

  if {$db_type!=""} {
    return [[lindex $db_types($db_type) 5] $handle]
  } else {
    return -code error "Handle $handle does not exist"
  }
}

proc db_colnames {handle} {
  global db_types
  global handles

  set db_type ""

  foreach i [array names handles] {
    if {$i==$handle} {
      set db_type $handles($i)
    }
  }

  if {$db_type!=""} {
    return [[lindex $db_types($db_type) 6] $handle]
  } else {
    return -code error "Handle $handle does not exist"
  }
}

proc db_coltypes {handle} {
  global db_types
  global handles

  set db_type ""

  foreach i [array names handles] {
    if {$i==$handle} {
      set db_type $handles($i)
    }
  }

  if {$db_type!=""} {
    return [[lindex $db_types($db_type) 7] $handle]
  } else {
    return -code error "Handle $handle does not exist"
  }
}

proc db_colsizes {handle} {
  global db_types
  global handles

  set db_type ""

  foreach i [array names handles] {
    if {$i==$handle} {
      set db_type $handles($i)
    }
  }

  if {$db_type!=""} {
    return [[lindex $db_types($db_type) 8] $handle]
  } else {
    return -code error "Handle $handle does not exist"
  }
}

proc db_tblsql {handle table} {
  db_sql $handle "select * from $table"
  set colnames [db_colnames $handle]
  set coltypes [db_coltypes $handle]
  if {[llength $colnames]!=[llength $coltypes]} {
    return -code error "names!=types"
  }
  set colsizes [db_colsizes $handle]
  if {[llength $colnames]!=[llength $colsizes]} {
    return -code error "names!=sizes"
  }
  append result "create table $table (\n"
  for {set i 0} {$i<[llength $colnames]} {incr i} {
    append result "  [lindex $colnames $i] [lindex $coltypes $i]([lindex $colsizes $i]),\n"
  }
  set result [string trimright $result ",\n"]
  append result ")"
}

proc db_copytable {handle table otherhandle} {
  set tblsql [db_tblsql $handle $table]
  db_sql $otherhandle $tblsql

  db_sql $handle "select * from $table"

  for {set row [db_next $handle]} {$row!=""} {set row [db_next $handle]} {
    set data [tcl2csv $row \' \,]

    db_sql $otherhandle "insert into $table values ($data)"
  }
}

proc tcl2csv {tclstring qc cc} {
  set result ""

  foreach i $tclstring {
    append result $qc$i$qc$cc
  }
  set result [string trimright $result $cc]
  return $result
}

proc csv2tcl {tclstring qc cc} {
  set result ""

  set blah [split $string $cc]
  set cont 0
  append exp $qc
  append exp {(.*)}
  append exp $qc

  for {set i 0} {$i<[llength $blah]} {incr i} {
    if {$cont==0} {
      set ent [lindex $blah $i]
    } else {
      append ent "$cc[lindex $blah $i]"
    }    
    if {[regexp $exp $ent blarg blarg]==1} { 
      lappend result $blarg
      set cont 0
    } else {
      set cont 1
    }
  }
  return $result
}

catch {package require MODDBC_Sybase}
catch {package require MODDBC_mSQL}
catch {package require MODDBC_Postgres}
